from enum import auto, Enum
from typing import Type
from requests import get, Response
from gmo_fx.response import Response as ResponseBase
from gmo_fx.urls import BASE_URL_PUBLIC


class Status(Enum):
    """
    外国為替FXの稼動状態
    """

    OPEN = auto()  # オープン
    CLOSE = auto()  # クローズ
    MAINTENANCE = auto()  # メンテナンス

    @classmethod
    def from_str(cls, text: str) -> Type["Status"]:
        match text:
            case "OPEN":
                return cls.OPEN
            case "CLOSE":
                return cls.CLOSE
            case "MAINTENANCE":
                return cls.MAINTENANCE
        raise ValueError(f"不明なステータスです。: {text}")


class StatusResponse(ResponseBase):
    status: Status

    def __init__(self, response: dict):
        super().__init__(response)
        self.status = Status.from_str(response["data"]["status"])


def get_status() -> StatusResponse:
    response: Response = get(f"{BASE_URL_PUBLIC}/status")
    if response.status_code == 200:
        response_json = response.json()
        return StatusResponse(response_json)

    raise RuntimeError(
        "ステータスが取得できませんでした。\n"
        f"status code: {response.status_code}\n"
        f"response: {response.text}"
    )



from dataclasses import dataclass
from datetime import datetime
from requests import get, Response
from gmo_fx.response import Response as ResponseBase
from gmo_fx.symbols import Symbol
from gmo_fx.urls import BASE_URL_PUBLIC


@dataclass
class Ticker:
    symbol: Symbol
    ask: float
    bid: float
    timestamp: datetime
    status: str


class TickerResponse(ResponseBase):
    tickers: list[Ticker]

    def __init__(self, response: dict):
        super().__init__(response)
        self.tickers = []

        data = response["data"]
        self.tickers = [
            Ticker(
                symbol=Symbol(d["symbol"]),
                ask=d["ask"],
                bid=d["bid"],
                status=d["status"],
                timestamp=datetime.strptime(d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"),
            )
            for d in data
        ]


def get_ticker() -> TickerResponse:
    response: Response = get(f"{BASE_URL_PUBLIC}/ticker")
    if response.status_code == 200:
        response_json = response.json()
        return TickerResponse(response_json)

    raise RuntimeError(
        "最新レートが取得できませんでした。\n"
        f"status code: {response.status_code}\n"
        f"response: {response.text}"
    )



from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Literal
from requests import get, Response
from gmo_fx.response import Response as ResponseBase
from gmo_fx.symbols import Symbol
from gmo_fx.urls import BASE_URL_PUBLIC


class KlineInterval(Enum):
    Min1 = "1min"
    Min5 = "5min"
    Min10 = "10min"
    Min15 = "15min"
    Min30 = "30min"
    H1 = "1hour"
    H4 = "4hour"
    H8 = "8hour"
    H12 = "12hour"
    D1 = "1day"
    W1 = "1week"
    M1 = "1month"


@dataclass
class Kline:
    open_time: datetime
    open: float
    high: float
    low: float
    close: float


class KlinesResponse(ResponseBase):
    klines: list[Kline]

    def __init__(self, response: dict):
        super().__init__(response)
        self.klines = []

        data = response["data"]
        self.klines = [
            Kline(
                open=float(d["open"]),
                high=float(d["high"]),
                low=float(d["low"]),
                close=float(d["close"]),
                open_time=datetime.fromtimestamp(int(d["openTime"]) / 1000),
            )
            for d in data
        ]


def get_klines(
    symbol: Symbol,
    price_type: Literal["BID", "ASK"],
    interval: KlineInterval,
    date: datetime,
) -> KlinesResponse:
    date_str = f"{date.year:04}"
    if interval in (
        KlineInterval.Min1,
        KlineInterval.Min5,
        KlineInterval.Min10,
        KlineInterval.Min15,
        KlineInterval.Min30,
        KlineInterval.H1,
    ):
        date_str += f"{date.month:02}{date.day:02}"
    base_url = f"{BASE_URL_PUBLIC}/klines"
    response: Response = get(
        f"{base_url}?"
        f"symbol={symbol.value}"
        f"&priceType={price_type}"
        f"&interval={interval.value}"
        f"&date={date_str}"
    )
    if response.status_code == 200:
        response_json = response.json()
        return KlinesResponse(response_json)

    raise RuntimeError(
        "Klineが取得できませんでした。\n"
        f"status code: {response.status_code}\n"
        f"response: {response.text}"
    )



from dataclasses import dataclass
from enum import Enum
from gmo_fx.response import Response as ResponseBase
from requests import get, Response
from gmo_fx.urls import BASE_URL_PUBLIC


class Symbol(Enum):
    USD_JPY = "USD_JPY"
    EUR_JPY = "EUR_JPY"
    GBP_JPY = "GBP_JPY"
    AUD_JPY = "AUD_JPY"
    NZD_JPY = "NZD_JPY"
    CAD_JPY = "CAD_JPY"
    CHF_JPY = "CHF_JPY"
    TRY_JPY = "TRY_JPY"
    ZAR_JPY = "ZAR_JPY"
    MXN_JPY = "MXN_JPY"
    EUR_USD = "EUR_USD"
    GBP_USD = "GBP_USD"
    AUD_USD = "AUD_USD"
    NZD_USD = "NZD_USD"


@dataclass
class Rule:
    symbol: Symbol
    min_open_order_size: int
    max_order_size: int
    size_step: int
    tick_size: float


class SymbolsResponse(ResponseBase):
    rules: list[Rule]

    def __init__(self, response: dict):
        super().__init__(response)
        self.rules = []

        data = response["data"]
        self.rules = [
            Rule(
                symbol=Symbol(d["symbol"]),
                min_open_order_size=int(d["minOpenOrderSize"]),
                max_order_size=int(d["maxOrderSize"]),
                size_step=int(d["sizeStep"]),
                tick_size=float(d["tickSize"]),
            )
            for d in data
        ]


def get_symbols() -> SymbolsResponse:
    response: Response = get(f"{BASE_URL_PUBLIC}/symbols")
    if response.status_code == 200:
        response_json = response.json()
        return SymbolsResponse(response_json)

    raise RuntimeError(
        "取引ルールが取得できませんでした。\n"
        f"status code: {response.status_code}\n"
        f"response: {response.text}"
    )



from unittest.mock import MagicMock, patch
from gmo_fx.status import get_status, Status

from tests.api_test_base import ApiTestBase


class TestStatusApi(ApiTestBase):

    @patch("gmo_fx.status.get")
    def test_status_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: get_status())

    @patch("gmo_fx.status.get")
    def test_status_open(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"status": "OPEN"})
        actual = get_status()
        assert actual.status == Status.OPEN

    @patch("gmo_fx.status.get")
    def test_status_close(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"status": "CLOSE"})
        actual = get_status()
        assert actual.status == Status.CLOSE

    @patch("gmo_fx.status.get")
    def test_status_maintenance(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"status": "MAINTENANCE"})
        actual = get_status()
        assert actual.status == Status.MAINTENANCE
